@@ -519,8 +519,7 @@ sub pp_list
     my @other_ops = ($pushmark_op);
 
     if (class($kid) eq 'NULL') {
-	return info_from_text($op, $self, '', 'list_null',
-			      {other_ops => \@other_ops});
+	return $self->info_from_string("list ''", $op, '', {other_ops => \@other_ops});
     }
     my $lop;
     my $local = "either"; # could be local(...), my(...), state(...) or our(...)
@@ -732,7 +731,7 @@ sub pp_const {
     my $self = shift;
     my($op, $cx) = @_;
     if ($op->private & OPpCONST_ARYBASE) {
-        return info_from_text($op, $self, '$[', 'const_ary', {});
+        return $self->info_from_string('const $[', $op, '$[');
     }
     # if ($op->private & OPpCONST_BARE) { # trouble with '=>' autoquoting
     # 	return $self->const_sv($op)->PV;
