@@ -14,7 +14,7 @@ tr/\x{345}/\x{370}/;
 # Constants in a block
 # CONTEXT no warnings;
 {
-    '???';
+    '?unrecoverable constant?';
     2;
 }
 ####
@@ -23,7 +23,7 @@ tr/\x{345}/\x{370}/;
 (1,2,3);
 0;
 >>>>
-'???', '???', '???';
+'?unrecoverable constant?', '?unrecoverable constant?', '?unrecoverable constant?';
 0;
 ####
 # Lexical and simple arithmetic
@@ -542,13 +542,13 @@ if (!GLIPP) { x() } elsif (!GLIPP) { z() } elsif (!GLIPP) { t() }
 >>>>
 x();
 x();
-'???';
+'?unrecoverable constant?';
 x();
 x();
 x();
 x();
 do {
-    '???'
+    '?unrecoverable constant?'
 };
 do {
     x()
@@ -565,11 +565,11 @@ do {
 do {
     x()
 };
-'???';
+'?unrecoverable constant?';
 do {
     t()
 };
-'???';
+'?unrecoverable constant?';
 !1;
 ####
 # TODO constant deparsing has been backed out for 5.12
@@ -1748,7 +1748,7 @@ my $foo;
 $_ = join $foo, pos
 >>>>
 my $foo;
-$_ = join('???', pos $_);
+$_ = join('?unrecoverable constant?', pos $_);
 ####
 # exists $a[0]
 our @a;
